@@ -13,22 +13,22 @@ if (getRversion() >= "2.15.1")
 # Conservation analysis ---------------------------------------------------
 read_structure <- function(file_names) {
   structure_names = c()
-  
+
   for (i in seq(1, length(file_names), by = 1)) {
     structure_names = append(structure_names, strsplit(file_names, "[.]")[[i]][1])
-    
+
   }
   structure_names_list = as.list(structure_names)
   structure_list = list()
-  
+
   i = 1
-  
+
   for (structure in file_names) {
     temp = read.table(structure)
     structure_list[[i]] = temp
-    
+
     i = i + 1
-    
+
   }
   names(structure_list) = structure_names
   return(structure_list)
@@ -43,14 +43,14 @@ delete_isoforms <- function(alignment) {
   # As input it takes the alignment file
   # Output: alignment without isoforms
   lines_to_delete = c()
-  
+
   pattern = "(-\\d+\\|)"
-  
+
   for (i in seq(1, length(alignment$nam))) {
     test = grep(perl = T,
                 pattern = pattern,
                 x = alignment$nam[i])
-    
+
     if (length(test) == 1) {
       lines_to_delete = append(lines_to_delete, i)
     }
@@ -85,14 +85,14 @@ consensus <-  function(alignment, threshold) {
     alignment_matrix = alignment
   }
   #alignment_matrix = as.matrix(as.character(alignment[[3]]))
-  
+
   count_cols = dim(alignment_matrix)[1]
   #count_cols = length(alignment_matrix)
   vec = seq(1, dim(alignment_matrix)[2], 1)
   #vec = seq(1, length(seqinr::s2c(alignment_matrix[1,])))
   len_of_vers = length(vec)
   mat = matrix("-", count_cols, len_of_vers)
-  
+
   for (i in seq(1, count_cols)) {
     #temp = seqinr::s2c(alignment_matrix[i])
     temp = alignment_matrix[i,]
@@ -100,7 +100,7 @@ consensus <-  function(alignment, threshold) {
       mat[i, j] = temp[j]
     }
   }
-  
+
   consens = c(rep("*", each = len_of_vers))
   for (j in vec) {
     alignment_col = mat[, j]
@@ -118,7 +118,7 @@ cons2seqs_ident <-  function(alignment, consensus_seq) {
   # number_of_seq- file[[1]]
   #consensus_seq - calculated consensus (output of consensusus())
   true_percentage = c()
-  
+
   for (i in seq(1, align_params(alignment)$row_no)) {
     true_percentage[i] = length(which((
       consensus_seq == seqinr::s2c(alignment$seq[i])
@@ -176,7 +176,7 @@ align_seq_mtx2grs <-
         }
       }
     }
-    
+
     if (grouping_method == 'polarity') {
       for (i in seq(1, rows)) {
         #Dividing AAs into groups of similar AAs in the matrices (according to the polarity and charges of the residues)
@@ -286,9 +286,9 @@ cons2seqs_sim <-
     col_no = dim(grouped_alignment)[2]
     aligned_sequences_matrixG = grouped_alignment
     consensusG = grouped_consensus_seq
-    
+
     true_percentage_G = c()
-    
+
     for (i in seq(from = 1, to = row_no, by = 1)) {
       true_percentage_G[i] = length(which((
         consensusG == (aligned_sequences_matrixG[i,])
@@ -303,13 +303,13 @@ alignment2matrix <- function(alignment) {
   #returns alignment as a matrix
   prmt = align_params(alignment = alignment)
   aligned_sequences_matrix = matrix("-", prmt$row_no, prmt$col_no)
-  
+
   for (i in seq(1, prmt$row_no)) {
     #Putting aligned seqs into matrix
     temp = toupper(seqinr::s2c(alignment$seq[i]))
     for (j in seq(1, prmt$col_no)) {
       aligned_sequences_matrix[i, j] = temp[j]
-      
+
     }
   }
   return(aligned_sequences_matrix)
@@ -342,22 +342,17 @@ calculate_AA_variation <-
         keyaas_treshold = prmt$row_no * (threshold / 100)
       }
     }
-    
+
     aligned_sequences_matrix = alignment2matrix(alignment = alignment)
     if (grouped == T) {
       aligned_sequences_matrix = align_seq_mtx2grs(aligned_sequences_matrix, grouping_method = grouping_method)
     }
     keyaas = matrix("n", dim(aligned_sequences_matrix)[2], 21 * 2)
     keyaas_per = matrix("n", dim(aligned_sequences_matrix)[2], 21 * 2)
-    pc = calculate_pseudo_counts(alignment)
     for (i in seq(1, dim(aligned_sequences_matrix)[2])) {
-      if (length(table(aligned_sequences_matrix[, i])) == 1) {
-        x = append(aligned_sequences_matrix[, i], "X")
-      } else{
-        x = aligned_sequences_matrix[, i]
-      }
-      table = (sort(table(x), decreasing = T))
+      table = (sort(table(aligned_sequences_matrix[, i]), decreasing = T))
       if (pseudo_counts) {
+        pc = calculate_pseudo_counts(alignment)
         df_table = as.data.frame(table)
         names(df_table) = c("AAs", "freq")
         pseudo_counts = pc[, i]
@@ -365,10 +360,16 @@ calculate_AA_variation <-
         df_pc = dplyr::mutate(df_pc, AAs = rownames(df_pc))
         df_merged_table = df_pc %>% dplyr::full_join(df_table)
         df_merged_table[is.na(df_merged_table)] = 0
-        df_merged_table = df_merged_table %>% dplyr::mutate(Frequency = pseudo_counts +
-                                                              freq) %>% dplyr::select(AAs, Frequency)
+        df_merged_table =
+          df_merged_table %>%
+          dplyr::mutate(Frequency = pseudo_counts + freq) %>%
+          dplyr::select(AAs, Frequency)
       } else{
-        df_merged_table = as.data.frame(table)
+        if(length(table) == 1){
+          df_merged_table = data.frame(col1=names(table), col2=table[1],row.names = 1)
+        }else{
+          df_merged_table = as.data.frame(table)
+        }
         names(df_merged_table) = c("AAs", "Frequency")
       }
       #AA types and frequencies
@@ -379,7 +380,7 @@ calculate_AA_variation <-
       order_decresasing = order(Frequency, decreasing = T)
       Frequency = Frequency[order_decresasing]
       AAs = AAs[order_decresasing]
-      
+
       keyaas[i, 1:length(matrix(AAs, 1, length(AAs)))] = matrix(AAs, 1, length(AAs))
       #if there are any AA (=always) then these are introduced to the keyaas matrix
       keyaas_per[i, 1:length(matrix(Frequency, 1, length(Frequency)))] = matrix(round(Frequency / sum(Frequency), 6) * 100, 1, length(AAs))
@@ -394,44 +395,44 @@ calculate_AA_variation <-
     #transpose matrix
     keyaas_per = t(keyaas_per[, 1:i]) #transpose matrix
     #merging key AAs symbols table with key AAs percentages table
-    
+
     size = dim(keyaas)
     output = matrix("-", size[1] * 2, size[2])
     if (!is.null(weights)) {
-      if (length(weights) != prmt$row_no) {
+      if (length(weights) != prmt$row_no)
         stop("The length of weights vector must equal the number of sequences in the alignment!")
-      } else{
-        weight = matrix("n",
-                        ncol =  dim(aligned_sequences_matrix)[2],
-                        nrow =  21)
-        weights = weights / mean(weights)
-        for (i in seq_len(ncol(aligned_sequences_matrix))) {
-          representatives = unique(keyaas[, i])
-          representatives = representatives[!representatives == "n"]
-          for (j in seq_len(length(representatives))) {
-            which_representative = which(aligned_sequences_matrix[, i] == representatives[j])
-            weight[j, i] = mean(weights[which_representative])
-          }
-          keyaas_per[, i] = as.numeric(keyaas_per[, i]) * as.numeric(weight[, i])
+      total_char = length(unique(c(aligned_sequences_matrix)))
+      if(total_char != 21)
+        warning("There are more than 20 AA types in your alignment.")
+      weight = matrix("n",
+                      ncol =  dim(aligned_sequences_matrix)[2],
+                      nrow =  total_char)
+      weights = weights / mean(weights)
+      for (i in seq_len(ncol(aligned_sequences_matrix))) {
+        representatives = unique(keyaas[, i])
+        representatives = representatives[!representatives == "n"]
+        for (j in seq_len(length(representatives))) {
+          which_representative = which(aligned_sequences_matrix[, i] == representatives[j])
+          weight[j, i] = mean(weights[which_representative])
+        }
+        a = weight[,i][which(weight[,i]!="n")]
+        b = keyaas_per[,i][which(keyaas_per[,i]!="n")]
+        keyaas_per[1:length(a), i] = as.numeric(b) * as.numeric(a)
         }
-        
       }
-    }
-    
     j = 1
     for (i in seq(1, size[1] * 2, 2)) {
       output[i, ] = keyaas[j, ]
       output[i + 1, ] = keyaas_per[j, ]
       j = j + 1
     }
-    
     return(list(
       AA = keyaas,
       percentage = keyaas_per,
       matrix = output
-      #weights = weight
     ))
   }
+
 noteworthy_seqs <- function(percentage, alignment) {
   max = which.max(percentage)
   namelist = alignment[[2]]
@@ -447,7 +448,7 @@ noteworthy_seqs <- function(percentage, alignment) {
     worst_consensus = out.min,
     most_common = out.common
   )
-  
+
   return(out)
 }
 convert_AA_symbol <- function(amino_acids) {
@@ -477,7 +478,7 @@ convert_AA_symbol <- function(amino_acids) {
       "Y",
       "V"
     )
-  
+
   names(map) <-
     c(
       "HIP",
@@ -504,7 +505,7 @@ convert_AA_symbol <- function(amino_acids) {
       "TYR",
       "VAL"
     )
-  
+
   sapply(strsplit(amino_acids, "(?<=[A-Z]{3})", perl = TRUE),
          function(x)
            paste(map[x], collapse = ""))
@@ -514,26 +515,26 @@ find_seqid <- function(sequence_id, library) {
   sequence_id = toupper(sequence_id)
   for (i in seq(1, length(library))) {
     ktory[i] = length(which((library[[i]] == sequence_id) == TRUE))
-    
+
     ktory1 = which(ktory == 1)
-    
+
   }
   seqid = library[[ktory1]][1]
-  
+
   return(seqid)
 }
 get_seq_names <- function(alignment) {
   names = alignment[[2]]
-  
+
   return(names)
 }
 find_seq <- function(sequence_id, alignment) {
   mat = as.matrix(as.character(alignment[[3]]))
-  
+
   nazwy_mat = get_seq_names(alignment)
-  
+
   which_uniprot = c()
-  
+
   for (i in seq(1, length(nazwy_mat))) {
     #finding indices of uniprot ID in seq
     which_uniprot[i] = length(
@@ -552,7 +553,7 @@ find_seq <- function(sequence_id, alignment) {
   seqs = mat[which(which_uniprot == 1)]
   #seq of protein uniprot ID
   seqs_character = length(which(seqinr::s2c(seqs) != "-") == T)
-  
+
   seq = list(sequence = seqs, len = seqs_character)
   return(seq)
 }
@@ -579,18 +580,18 @@ barplotshow <- function (position, AA_variation)
   x_val = round(as.numeric(AA_variation$percentage[seq(1,
                                                        row), position]), digits = 1)
   x_val = format(x_val, nsmall = 1)
-  
+
   x_val_raw = as.numeric(AA_variation$percentage[seq(1,
                                                      row), position])
-  
+
   ind = order(-x_val_raw)
-  
+
   x_names = x_names[ind]
   x_val = x_val[ind]
   x_val_raw = x_val_raw[ind]
-  
-  
-  
+
+
+
   plot = barplot(
     x_val_raw,
     names.arg = x_names,
@@ -607,7 +608,7 @@ barplotshow <- function (position, AA_variation)
     main = bquote(Amino ~ acid ~ variation ~ on ~ the ~ .(position) ~ position),
     col = "lightgrey"
   )
-  
+
   z <- function(s) {
     if (s == "0.0") {
       "<0.1"
@@ -615,9 +616,9 @@ barplotshow <- function (position, AA_variation)
       s
     }
   }
-  
+
   x_val_raw <- x_val_raw + 9
-  
+
   text(
     x = plot,
     y = x_val_raw,
@@ -678,7 +679,7 @@ get_pos_based_seq_weights <- function(alignment, gap = TRUE, normalized = TRUE){
   align_param = align_params(alignment)
   weights_mtx = matrix(NA,nrow = align_param$row_no, ncol = align_param$col_no)
   alignment_mtx = alignment2matrix(alignment)
-  
+
   if (gap == TRUE){
     # treat gap as other residues
     for(i in seq(1, ncol(alignment_mtx))){
@@ -687,19 +688,19 @@ get_pos_based_seq_weights <- function(alignment, gap = TRUE, normalized = TRUE){
         weights_mtx[j,i] = 1/(length(temp_table)*temp_table[alignment_mtx[j,i]])
       }
     }
-    
+
     # chcecking if weights should be normalized
     if(normalized ==TRUE){
       weights = apply(weights_mtx,MARGIN = 1, sum)/align_param$col_no
-      
+
     }else{
       weights = apply(weights_mtx,MARGIN = 1, sum)
     }
     return(weights)
-    
-    
+
+
   }else{
-    
+
     # skip gaps
     for(i in seq(1, ncol(alignment_mtx))){
       temp_table <- table(alignment_mtx[,i])
@@ -711,19 +712,19 @@ get_pos_based_seq_weights <- function(alignment, gap = TRUE, normalized = TRUE){
           weights_mtx[j,i] = 0
         } else{
           weights_mtx[j,i] = 1/(length(temp_table)*temp_table[alignment_mtx[j,i]])
-          
+
         }
       }
     }
     # chcecking if weights should be normalized
     if(normalized ==TRUE){
       weights = apply(weights_mtx,MARGIN = 1, sum)/align_param$col_no
-      
+
     }else{
       weights = apply(weights_mtx,MARGIN = 1, sum)
     }
     return(weights)
-    
+
   }
 }
 
@@ -754,11 +755,11 @@ create_final_CSV <-
                           nrow = length(score_list),
                           ncol = length(score_list[[1]]))
       scores_mtx_names = c()
-      
+
       for (i in seq(1, length(score_list))) {
         scores_mtx[i,] = score_list[[i]]
         scores_mtx_names[i] = names(score_list)[i]
-        
+
       }
       rownames(scores_mtx) = scores_mtx_names
       alignment_position = seq(1, dim(variations_matrix$matrix)[2], by = 1)
@@ -769,7 +770,7 @@ create_final_CSV <-
         structure_output,
         scores_mtx
       )
-      
+
     }
     files_no = ceiling(dim(final_output)[2] / 1000)
     for (i in seq(1, files_no, 1)) {
@@ -816,22 +817,22 @@ Escore_conservativity <-
       )
     }
     max_cons = c()
-    
+
     for (i in seq(1, length(var_aa$matrix[1, ]), 1)) {
       if (is.na(as.numeric(var_aa$matrix[2, i])) == FALSE) {
         max_cons[i] = as.numeric(var_aa$matrix[2, i])
       }
     }
     AA = which(max_cons != 0)
-    
+
     ile_var = c()
-    
+
     for (i in seq(1, dim(var_aa$AA)[2], 1)) {
       ile_var[i] = length(which(var_aa$AA[, i] != "n" &
                                   var_aa$AA[, i] != "-"))
     }
     pre_conservativity = max_cons / ile_var
-    
+
     pre_conservativity[which(is.na(pre_conservativity))] = 0
     part_con = pre_conservativity
     part_con[which(is.infinite(part_con))] = 0
@@ -840,7 +841,7 @@ Escore_conservativity <-
     E[which(is.infinite(E))] = 0 # change Infs to 0
     E_score = (E / max(E))
     return_data = E_score
-    
+
     return(return_data)
   }
 
@@ -854,7 +855,7 @@ kabat_conservativity <-
       aligned_sequences_matrix = alignment
     }
     Kabat = rep(NaN, dim(aligned_sequences_matrix)[2])
-    
+
     var_aa = suppressWarnings(
       calculate_AA_variation(alignment, weights = weights, pseudo_counts = pseudo_counts)
     )
@@ -880,7 +881,7 @@ kabat_conservativity <-
       Kabat[rep] = (K / n1) * N
     }
     Kabat_entropy_normalized = Kabat / max(Kabat)
-    
+
     return(Kabat_entropy_normalized)
   }
 
@@ -943,7 +944,7 @@ shannon_conservativity <-
         pseudo_counts = pseudo_counts
       )
     )
-    
+
     for (rep in seq(1, dim(aligned_sequences_matrix)[2], 1)) {
       column = aligned_sequences_matrix[, rep]
       AAs = var_aa$AA[, rep]
@@ -978,7 +979,7 @@ pairwise_alignment_MSA <- function(alignment) {
     seqj = seqinr::c2s(seqj[seqj != "-"])
     return(Biostrings::pairwiseAlignment(seqi, seqj, scoreOnly = T))
   }
-  
+
   score_mtx = matrix(NA, nrow = alignment$nb, ncol = alignment$nb)
   for (i in seq_len(alignment$nb)) {
     seqi = alignment$seq[i]
@@ -1018,7 +1019,7 @@ CRE_conservativity <-
         stop("You need to specify the hmmbuild path as it cannot be located in your system")
       }
     }
-    
+
     #perform hierarchical clustering on the distance matrix obtained from pairwise alignemnt matrix
     dendro = stats::hclust(stats::dist(pairwiseAlignemnt_scores), method = "average")
     #cut the tree and get clusters
@@ -1119,7 +1120,7 @@ D_matrix <- function(substitution_matrix) {
   #CREATE DISTANCE MATRIX
   for (i in 1:k) {
     index = i
-    
+
     for (j in 1:k) {
       if (i != j) {
         distance[i, j] = (values[i, i] - values[i, j] / values[i, i])
@@ -1127,7 +1128,7 @@ D_matrix <- function(substitution_matrix) {
       if (i == j) {
         distance[i, j] <- 0
       }
-      
+
     }
   }
   output = list(substitution_matrix[[1]], distance)
@@ -1145,7 +1146,7 @@ landgraf_conservativity <-
     dissim_mtx = D_matrix(pre_dissim_mtx)
     conservation = rep(NaN, dim(aligned_sequences_matrix)[2])
     status = 0
-    
+
     pb <- progress_bar$new(
       format = paste("Landgraf calculation", " [:bar] :percent eta: :eta"),
       total = dim(aligned_sequences_matrix)[2],
@@ -1155,15 +1156,15 @@ landgraf_conservativity <-
     for (rep in seq(1, dim(aligned_sequences_matrix)[2], 1)) {
       pb$tick()
       column = aligned_sequences_matrix[, rep]
-      
+
       values = as.numeric(as.matrix(dissim_mtx[[2]]))
       alpha = dissim_mtx[[1]]
       dim(values) <- dim(dissim_mtx[[2]])
       iterator = seq(1:length(column))
       sum_of_dist = 0
-      
+
       global_sum = 0
-      
+
       for (i in iterator) {
         iterator2 = c((i + 1):length(column))
         posa = match(column[i], alpha)
@@ -1268,7 +1269,7 @@ create_structure_seq <-
     if (length(missing$values) == 0) {
       missing = NULL
     }
-    
+
     #finds an uniprot name from alignent- extract uniprot seq from alignment
     base_seq = find_seq(sequence_id, alignment)
     structure_probabilities = list()
@@ -1280,7 +1281,7 @@ create_structure_seq <-
         structures_probability = as.vector(structure_list[[i]][[3]])
         structures_probability = structures_probability[-1]
         structures_probability = structures_probability / max(structures_probability)# normalization
-        
+
       } else{
         prob_data = F
         structures_probability = NULL
@@ -1288,11 +1289,11 @@ create_structure_seq <-
       structures_indices = structures_indices[-1]
       structure_idx = as.numeric(structures_indices)
       structures_names = structures_names[-1]
-      
+
       if (!is.null(shift)) {
         structure_idx = structure_idx + rep(shift, length(structure_idx))
       }
-      
+
       if (length(missing) != 0) {
         for (z in seq(1, length(missing$values))) {
           for (l in seq(1, length(structure_idx))) {
@@ -1420,10 +1421,10 @@ get_structures_idx <- function(structure) {
   stru_index = list()
   for (i in seq(1:length(structure[, 1]))) {
     stru_index[[i]] = which(structure[i, ] == "S")
-    
+
   }
   whole_prot = which(structure[1, ] != "-")
-  
+
   out = list(proteinIndices = whole_prot, structureIndices = stru_index)
   names(out[[2]]) = rownames(structure)
   return(out)
@@ -1551,8 +1552,8 @@ plot_structure_on_protein <-
     #protein entropy- list of an entropy values (with different scores)
     #structure_profils - list of entropy scores for structures (as list) where [[1]] entropy value
     # and [[2]] index in protein
-    
-    
+
+
     if (length(protein_entropy) == length(structure_profiles[[1]][[1]][, 1])) {
       score_names = names(protein_entropy)
       prot_lenght = length(protein_entropy[[1]])
@@ -1568,7 +1569,7 @@ plot_structure_on_protein <-
         for (i in seq(1, StruLen)) {
           #structure_names[i] = paste("stru", i)
           structure_names[i] = names(structure_profiles)[i]
-          
+
         }
       }
       if (is.null(legend_pos)) {
@@ -1576,7 +1577,7 @@ plot_structure_on_protein <-
       }
       for (i in seq(1, length(score_names))) {
         if (isRStudio <- Sys.getenv("RSTUDIO") == "1") {
-          
+
         } else{
           #X11()
           dev.new()
@@ -1713,7 +1714,7 @@ compare_cons_metrics <-
         }
       }
     }
-    
+
   }
 
 kolmogorov_smirnov_test <-
@@ -1729,9 +1730,9 @@ kolmogorov_smirnov_test <-
     metrics_count = length(protein_entropy)
     structures_count = length(structure_entropy)
     structure_names = names(structure_entropy)
-    
+
     alt_hip = c("two.sided", "less", "greater")[alternative]
-    
+
     pval_mtx = matrix(NA, nrow = metrics_count, ncol = structures_count)
     rownames(pval_mtx) <- names(protein_entropy)
     colnames(pval_mtx) <- structure_names
@@ -1742,7 +1743,7 @@ kolmogorov_smirnov_test <-
         pval_mtx[i, j] = stats::ks.test(reference, temp, alternative = alt_hip)$p.value
       }
     }
-    
+
     if (make_plot == T) {
       colors = rainbow(structures_count)
       for (i in seq(1, metrics_count)) {
